// UC: 21020 - Computação Gráfica (2022/23)
// Aluno: 1902551 - Pedro Gaspar

import * as THREE from "https://unpkg.com/three@0.124.0/build/three.module.js";

/**
 * Calcula a posição no instante t de uma curva Bézier quártica, segundo a função
 * f(t) = (1 - t)^4 * c0 + 4 * (1 - t)^3 * t * c1 + 6 * (1 - t)^2 * t^2 * c2 + 4 * (1 - t) * t^3 * c3 + t^4 * c4
 * @param {Vector3} c0 Ponto 1
 * @param {Vector3} c1 Ponto 2
 * @param {Vector3} c2 Ponto 3
 * @param {Vector3} c3 Ponto 4
 * @param {Vector3} c4 Ponto 5
 * @param {number} t Instante de tempo, compreendido entre 0 e 1
 * @returns Vetor com a posição no instante t da curva Bézier para
 * os pontos indicados
 */
const bezier4 = (c0, c1, c2, c3, c4, t) => {
  if (t < 0 || t > 1) {
    throw new Error("O valor de t deve estar entre 0 e 1");
  }

  const t2 = t * t;
  const t3 = t2 * t;
  const t4 = t3 * t;

  const s = 1 - t;
  const s2 = s * s;
  const s3 = s2 * s;
  const s4 = s3 * s;

  const x =
    s4 * c0.x +
    4 * s3 * t * c1.x +
    6 * s2 * t2 * c2.x +
    4 * s * t3 * c3.x +
    t4 * c4.x;
  const y =
    s4 * c0.y +
    4 * s3 * t * c1.y +
    6 * s2 * t2 * c2.y +
    4 * s * t3 * c3.y +
    t4 * c4.y;
  const z =
    s4 * c0.z +
    4 * s3 * t * c1.z +
    6 * s2 * t2 * c2.z +
    4 * s * t3 * c3.z +
    t4 * c4.z;

  return new THREE.Vector3(x, y, z);
};

export { bezier4 };
