// UC: 21020 - Computação Gráfica (2022/23)
// Aluno: 1902551 - Pedro Gaspar

import * as THREE from "https://unpkg.com/three@0.124.0/build/three.module.js";
import { bezier4 } from "../bezier4.mjs";

/**
 * Classe wrapper do THREE.Curve para a nossa curva de Bézier.
 * Adaptado do exemplo em https://threejs.org/docs/#api/en/geometries/TubeGeometry
 */
export class BezierCurve extends THREE.Curve {
  points;

  constructor(points, scale = 1) {
    super();

    // Aqui recebemos os pontos a utilizar na curva. Estes pontos correspondem
    // às posições x,y,z das bolas no plano.
    this.points = points;
    this.scale = scale;
  }

  getPoint(t, optionalTarget = new THREE.Vector3()) {
    const bezierVector = bezier4(
      new THREE.Vector3(this.points[0].x, this.points[0].y, this.points[0].z),
      new THREE.Vector3(this.points[1].x, this.points[1].y, this.points[1].z),
      new THREE.Vector3(this.points[2].x, this.points[2].y, this.points[2].z),
      new THREE.Vector3(this.points[3].x, this.points[3].y, this.points[3].z),
      new THREE.Vector3(this.points[4].x, this.points[4].y, this.points[4].z),
      t
    );

    return optionalTarget
      .set(bezierVector.x, bezierVector.y, bezierVector.z)
      .multiplyScalar(this.scale);
  }
}
