// UC: 21020 - Computação Gráfica (2022/23)
// Aluno: 1902551 - Pedro Gaspar

/**
 * Gera um número aleatório num intervalo com a precisão especificada
 * @param {number} min Valor mínimo a gerar
 * @param {number} max Valor máximo a gerar
 * @param {number} decimalCases Número de casas decimais (precisão) do número
 * @returns Número aleatório no intervalo especificado
 */
export function randomFloat(min, max, decimalCases = 2) {
  const random = Math.random() * (max - min) + min;
  return Number(random.toFixed(decimalCases));
}

/**
 * Faz com que uma função apenas seja executada no mínimo, uma vez a cada timeFrame milissegundos.
 * Na primeira execução, a função é executada de imediato.
 * @param {*} func Função a ser executada após o tempo
 * @param {*} timeFrame Intervalo mínimo de tempo entre execuções
 */
export function throttle(func, timeFrame) {
  var lastTime = 0;
  return function (...args) {
    var now = new Date();
    if (now - lastTime >= timeFrame) {
      func(...args);
      lastTime = now;
    }
  };
}

/**
 * Arrendonda um número para um número de casas decimais especificado
 * @param {*} num Número a arredondar
 * @param {*} decimalCases Número de casas decimais que o número arrondado deve ter
 * @returns Número arredondado às casas decimais fornecidas
 */
export function round(num, decimalCases = 2) {
  return parseFloat(num.toFixed(decimalCases));
}
